UNIT dio;

{Device Input/Output Unit for TP/BP.

 Released "AS IS" to the public domain - use at your own risk!

 It provides routines for communicating with Device Drivers (if you don't
 know what that means, then this unit is probably of no use to you).

 Devices are handled here as type "text" (that's what the 'T' stands for in
 the routine names). However, you should be able to create binary versions
 of these routines, by simply defining a different (binary) file type and
 substituting this for "text" in the routine headers, and also substituting
 "FileRec" for "TextRec" where this occurs. (Oh - and rename the routines,
 to avoid confusion).

 Joe da Silva}

INTERFACE

Uses
   dos;

Procedure IOCtlT (var f: text; ctlstr : string);

Procedure RawModeT (var f: text);

Procedure IOWriteT (var f: text; wrstr : string);

Procedure IOReadT (var f: text; var rdstr : string);

IMPLEMENTATION

Procedure IOCtlT (var f: text; ctlstr : string);
{Note: "registers" and "TextRec" are defined in "dos" unit.}
const
   IOControl = $44;
   CtlWrite = 3;
   CarryMask = $0001;
var
   regs : registers;
begin {IOCtlT}
   regs.DS := seg(ctlstr[1]);
   regs.DX := ofs(ctlstr[1]);
   regs.CX := ord(ctlstr[0]);
   regs.BX := TextRec(f).Handle;
   regs.AL := CtlWrite;
   regs.AH := IOControl;
   msdos(regs);
   if (regs.Flags and CarryMask) <> 0
   then
      begin {then}
         writeln('Error in I/O Control call (',ctlstr,').');
         halt(regs.AX)
      end {then}
end; {IOCtlT}

Procedure RawModeT (var f: text);
{Sets device into "raw" mode, preventing DOS from trying to
 interpret any special characters during data transactions.
 Note: "registers" and "TextRec" are defined in "dos" unit.}
const
   IOControl = $44;
   GetInfo = 0;
   SetInfo = 1;
   RawMask = $20;
   CarryMask = $0001;
var
   regs : registers;
begin {RawModeT}
   regs.BX := TextRec(f).Handle;
   regs.AL := GetInfo;
   regs.AH := IOControl;
   msdos(regs);
   if (regs.Flags and CarryMask) <> 0
   then
      begin {then}
         writeln('Error in I/O Control GetInfo (',regs.AX,').');
         halt(regs.AX)
      end; {then}
   regs.DL := regs.DL or RawMask;
   regs.DH := 0; {Call requirement}
   regs.BX := TextRec(f).Handle;
   regs.AL := SetInfo;
   regs.AH := IOControl;
   msdos(regs);
   if (regs.Flags and CarryMask) <> 0
   then
      begin {then}
         writeln('Error in I/O Control SetInfo (',regs.AX,').');
         halt(regs.AX)
      end {then}
end; {RawModeT}

Procedure IOWriteT (var f: text; wrstr : string);
{Note: "registers" and "TextRec" are defined in "dos" unit.}
const
   IOWrite = $40;
   CarryMask = $0001;
var
   regs : registers;
begin {IOWriteT}
   regs.DS := seg(wrstr[1]);
   regs.DX := ofs(wrstr[1]);
   regs.CX := ord(wrstr[0]);
   regs.BX := TextRec(f).Handle;
   regs.AH := IOWrite;
   msdos(regs);
   if (regs.Flags and CarryMask) <> 0
   then
      begin {then}
         writeln('Error in I/O Write call (',wrstr,').');
         halt(regs.AX)
      end {then}
end; {IOWriteT}

Procedure IOReadT (var f: text; var rdstr : string);
{Note: "registers" and "TextRec" are defined in "dos" unit.}
const
   IORead = $3F;
   CarryMask = $0001;
   MaxString = 255;
var
   regs : registers;
begin {IOReadT}
   regs.DS := seg(rdstr[1]);
   regs.DX := ofs(rdstr[1]);
   regs.CX := MaxString;
   regs.BX := TextRec(f).Handle;
   regs.AH := IORead;
   msdos(regs);
   if (regs.Flags and CarryMask) = 0
   then
      rdstr[0] := chr(regs.AL)
   else
      begin {else}
         writeln('Error in I/O Read call (',regs.AX,').');
         halt(regs.AX)
      end {else}
end; {IOReadT}

END. {dio}
